@@ -1,7 +1,7 @@
 /*
- * $Id: di.h,v 1.54 2010-12-31 14:29:54-08 bll Exp $
+ * $Id: di.h,v 1.57 2011-10-03 20:14:37-07 bll Exp $
  * $Source: /home/bll/DI/di/C/RCS/di.h,v $
- * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
+ * Copyright 1994-2011 Brad Lanam, Walnut Creek, CA
  */
 
 #ifndef __INC_DI_H_
@@ -99,10 +99,12 @@
   typedef unsigned long long _fs_size_t;
   typedef long long _s_fs_size_t;
 # define DI_LL "ll"
+# define DI_LLu "llu"
 #else
   typedef unsigned long _fs_size_t;
   typedef long _s_fs_size_t;
 # define DI_LL "l"
+# define DI_LLu "lu"
 #endif
 #if _siz_long_double >= 8
   typedef long double _print_size_t;
@@ -181,12 +183,6 @@ typedef struct
 #if ! _dcl_errno
   extern int errno;
 #endif
-#if ! _dcl_optind
-  extern int optind;
-#endif
-#if ! _dcl_optarg
-  extern char *optarg;
-#endif
 
 /* workaround for AIX - mntctl not declared */
 # if _lib_mntctl && _npt_mntctl
@@ -199,14 +195,6 @@ extern int  di_getDiskEntries       _((diDiskInfo_t **, int *));
 extern void di_getDiskInfo          _((diDiskInfo_t **, int *));
  /* diquota.c */
 extern void diquota                 _((diQuota_t *));
- /* getopt.c */
-# if ! _lib_getopt || _npt_getopt
-#  if ! _lib_getopt
-extern char *optarg;
-extern int optind;
-#  endif
-extern int getopt                   _((int, char * const [], const char *));
-# endif
  /* strdup.c */
 # if ! _lib_strdup
 extern char *strdup                 _((const char *));
