@@ -1,7 +1,7 @@
 /*
- * $Id: digetentries.c,v 1.31 2010-12-31 14:30:17-08 bll Exp $
+ * $Id: digetentries.c,v 1.36 2011-10-04 16:52:07-07 bll Exp $
  * $Source: /home/bll/DI/di/C/RCS/digetentries.c,v $
- * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
+ * Copyright 1994-2011 Brad Lanam, Walnut Creek, CA
  */
 
 /********************************************************/
@@ -82,7 +82,7 @@
 #if _sys_fs_types               /* OSF/1, AROS */
 # include <sys/fs_types.h>
 #endif
-#if _sys_mnttab                 /* Solaris */
+#if _sys_mnttab                 /* Solaris, SCO_SV, UnixWare */
 # include <sys/mnttab.h>        /* getmntent(); MNTTAB */
 #endif
 
@@ -159,7 +159,7 @@
 #endif
 
 #if (_lib_getmntent || \
-    _statfs_args > 0) && \
+    _args_statfs > 0) && \
     ! _lib_getmntinfo && \
     ! _lib_getfsstat && \
     ! _lib_getvfsstat && \
@@ -356,7 +356,7 @@ di_getDiskEntries (diskInfo, diCount)
 
     if (debug > 0) { printf ("# getDiskEntries: set/get/endmntent\n"); }
 /* if both are set not an ansi compiler... */
-#if _setmntent_args == 1
+#if _args_setmntent == 1
     if ((f = setmntent (DI_MOUNT_FILE)) == (FILE *) NULL)
 #else
     if ((f = setmntent (DI_MOUNT_FILE, "r")) == (FILE *) NULL)
@@ -475,7 +475,7 @@ di_getQNXDiskEntries (ipath, diskInfo, diCount)
 {
     diDiskInfo_t    *diptr;
     int             idx;
-    char            path [MAXPATHLEN];
+    char            path [MAXPATHLEN + 1];
     int             len;   /* current length of path */
     DIR             *dirp;
     struct dirent   *dent;
@@ -493,7 +493,7 @@ di_getQNXDiskEntries (ipath, diskInfo, diCount)
       return 0;
     }
 
-    strcpy (path, ipath);
+    strncpy (path, ipath, MAXPATHLEN);
     len = strlen (path);
 
     if (!(dirp = opendir(path))) {
@@ -513,7 +513,7 @@ di_getQNXDiskEntries (ipath, diskInfo, diCount)
       }
 
       path[len] = '/';
-      strcpy (&path[len+1], dent->d_name);
+      strncpy (&path[len+1], dent->d_name, MAXPATHLEN - len - 1);
       if (debug > 4) { printf ("check: %s\n", path); }
 
       memset(&statinfo, 0, sizeof(statinfo));
@@ -562,7 +562,7 @@ di_getQNXDiskEntries (ipath, diskInfo, diCount)
       strncpy (diptr->special, tspecial, DI_SPEC_NAME_LEN);
       strncpy (diptr->name, path + 11, DI_NAME_LEN);
       if (*diptr->name == '\0') {
-        strcpy (diptr->name, "/");
+        strncpy (diptr->name, "/", DI_NAME_LEN);
       }
       if (debug > 4) { printf ("found: %s %s\n", diptr->special, diptr->name); }
     }
@@ -660,7 +660,7 @@ di_getDiskEntries (diskInfo, diCount)
 
 
 #if _lib_getfsstat && \
-    ! _lib_getvfsstat
+    ! (_lib_getvfsstat && _args_getvfsstat == 3)
 
 /*
  * di_getDiskEntries
@@ -691,18 +691,18 @@ di_getDiskEntries (diskInfo, diCount)
 # if _dcl_mnt_names && _mem_struct_statfs_f_type
     short           fstype;
 # endif
-    _getfsstat_type bufsize;
+    _c_arg_2_getfsstat bufsize;
     struct statfs   *mntbufp;
     struct statfs   *sp;
 
     if (debug > 0) { printf ("# getDiskEntries: getfsstat\n"); }
-    count = getfsstat ((struct statfs *) NULL, (_getfsstat_type) 0, MNT_NOWAIT);
+    count = getfsstat ((struct statfs *) NULL, (_c_arg_2_getfsstat) 0, MNT_NOWAIT);
     if (count < 1)
     {
         fprintf (stderr, "Unable to do getfsstat () errno %d\n", errno);
         return -1;
     }
-    bufsize = (_getfsstat_type) (sizeof (struct statfs) * (Size_t) count);
+    bufsize = (_c_arg_2_getfsstat) (sizeof (struct statfs) * (Size_t) count);
     mntbufp = malloc ((Size_t) bufsize);
     memset ((char *) mntbufp, '\0', sizeof (struct statfs) * (Size_t) count);
     count = getfsstat (mntbufp, bufsize, MNT_NOWAIT);
@@ -1100,7 +1100,7 @@ di_getDiskEntries (diskInfo, diCount)
 
 #endif /* _lib_getmntinfo */
 
-#if _lib_getvfsstat
+#if _lib_getvfsstat && _args_getvfsstat == 3
 
 /*
  * di_getDiskEntries
@@ -1219,6 +1219,10 @@ di_getDiskEntries (diskInfo, diCount)
 
 #if _lib_getmnt
 
+# if _npt_getmnt
+  int getmnt _((int *, struct fs_data *, int, int, char *));
+# endif
+
 /*
  * di_getDiskEntries
  *
@@ -1241,7 +1245,7 @@ di_getDiskEntries (diskInfo, diCount)
 {
     diDiskInfo_t     *diptr;
     int             count;
-    Size_t          bufsize;
+    int             bufsize;
     int             idx;
     short           fstype;
     struct fs_data  *fsdbuf;
@@ -1251,7 +1255,7 @@ di_getDiskEntries (diskInfo, diCount)
 
     if (debug > 0) { printf ("# getDiskEntries: getmnt\n"); }
     bufsize = NMOUNT * sizeof (struct fs_data);  /* enough for max # mounts */
-    fsdbuf = (struct fs_data *) malloc (bufsize);
+    fsdbuf = (struct fs_data *) malloc ((Size_t) bufsize);
     if (fsdbuf == (struct fs_data *) NULL)
     {
         fprintf (stderr, "malloc (%d) for getmnt () failed errno %d\n",
@@ -1585,11 +1589,11 @@ di_getDiskEntries (diskInfo, diCount)
         {
           DWORD br;
           BOOL bSuccess;
-          char handleName [MSDOS_BUFFER_SIZE];
+          char handleName [MSDOS_BUFFER_SIZE + 1];
 
           diptr->printFlag = diskflag;
           bSuccess = 1;
-          strcpy (handleName, "\\\\.\\");
+          strncpy (handleName, "\\\\.\\", MSDOS_BUFFER_SIZE);
           strncat (handleName, p, MSDOS_BUFFER_SIZE);
           handleName[strlen(handleName)-1] = '\0';
           HANDLE hDevice = CreateFile (handleName,
