@@ -1,7 +1,7 @@
 /*
- * $Id: dimntopt.h,v 1.7 2010-01-31 21:09:21-08 bll Exp $
- * $Source: /home/bll/DI/RCS/dimntopt.h,v $
- * Copyright 2010 Brad Lanam, Walnut Creek, CA
+ * $Id: dimntopt.h,v 1.10 2011-09-29 11:52:36-07 bll Exp $
+ * $Source: /home/bll/DI/di/C/RCS/dimntopt.h,v $
+ * Copyright 2011 Brad Lanam, Walnut Creek, CA
  */
 
 #ifndef __INC_DIMNTOPT_H_
@@ -19,15 +19,15 @@
 # define _DI_INC_SYS_MOUNT 1
 # include <sys/mount.h>             /* MNT_...; M_... (hp-ux) */
 #endif
-#if _sys_mntent                     /* Solaris */
+#if _sys_mnttab                     /* SCO_SV, UnixWare */
+# include <sys/mnttab.h>            /* required for mntent.h */
+#endif
+#if _sys_mntent                     /* Solaris, SCO_SV, UnixWare */
 # include <sys/mntent.h>            /* MNTOPT_... */
 #endif
 #if _sys_fstypes                    /* NetBSD */
 # include <sys/fstypes.h>
 #endif
-#if _sys_fs_types                   /* OSF/1 */
-# include <sys/fs_types.h>
-#endif
 #if _sys_vmount                     /* AIX */
 # include <sys/vmount.h>            /* MNT_... */
 #endif
