@@ -1,8 +1,8 @@
 /*
- * $Id: diquota.c,v 1.18 2010-12-31 14:30:51-08 bll Exp $
+ * $Id: diquota.c,v 1.24 2011-10-04 16:52:45-07 bll Exp $
  * $Source: /home/bll/DI/di/C/RCS/diquota.c,v $
  *
- * Copyright 2010 Brad Lanam, Walnut Creek, CA
+ * Copyright 2011 Brad Lanam, Walnut Creek, CA
  */
 
 #include "config.h"
@@ -69,7 +69,7 @@
   extern "C" {
 #endif
 
-#if _hdr_rpc_rpc && _hdr_rpcsvc_rquota
+#if _has_std_nfs_quotas
 static bool_t xdr_quota_get _((XDR *, struct getquota_args *));
 static bool_t xdr_quota_rslt _((XDR *, struct getquota_rslt *));
 static void diquota_nfs _((diQuota_t *));
@@ -121,6 +121,11 @@ diquota (diqinfo)
   int               xfsflag;
 #if _typ_struct_dqblk
   struct dqblk      qinfo;
+#endif
+#if _typ_struct_ufs_dqblk
+  struct ufs_dqblk  qinfo;
+#endif
+#if _typ_struct_dqblk || _typ_struct_ufs_dqblk
   char              *qiptr;
 #endif
 #if _typ_fs_disk_quota_t
@@ -144,7 +149,7 @@ diquota (diqinfo)
 
   if (strncmp (diqinfo->type, "nfs", (Size_t) 3) == 0 &&
       strcmp (diqinfo->type, "nfsd") != 0) {
-#if _hdr_rpc_rpc && _hdr_rpcsvc_rquota
+#if _has_std_nfs_quotas
     diquota_nfs (diqinfo);
 #endif
     return;
@@ -167,7 +172,7 @@ diquota (diqinfo)
     ucmd = QCMD (Q_GETQUOTA, USRQUOTA);
     gcmd = QCMD (Q_GETQUOTA, GRPQUOTA);
 #endif
-#if _typ_struct_dqblk
+#if _typ_struct_dqblk || _typ_struct_ufs_dqblk
     qiptr = (char *) &qinfo;
 #endif
   }
@@ -180,13 +185,13 @@ diquota (diqinfo)
 #endif
 
   if (debug > 5) {
-    printf ("quota: quotactl on %s (%d)\n", diqinfo->name, _quotactl_pos);
+    printf ("quota: quotactl on %s (%d %d)\n", diqinfo->name, _quotactl_pos_1, _quotactl_pos_2);
   }
-#if _lib_quotactl && _quotactl_pos == 1
+#if _lib_quotactl && _quotactl_pos_1
   rc = quotactl (diqinfo->name, ucmd,
         (int) diqinfo->uid, (caddr_t) qiptr);
 #endif
-#if _lib_quotactl && _quotactl_pos == 2
+#if _lib_quotactl && _quotactl_pos_2
 # if _AIX  /* AIX has linux compatibility routine, but still need name */
   rc = quotactl (ucmd, diqinfo->name,
         (int) diqinfo->uid, (caddr_t) qiptr);
@@ -199,13 +204,13 @@ diquota (diqinfo)
   {
     int             fd;
     struct quotctl  qop;
-    char            tname [DI_NAME_LEN];
+    char            tname [DI_NAME_LEN + 1];
 
     qop.op = Q_GETQUOTA;
     qop.uid = diqinfo->uid;
     qop.addr = (caddr_t) qiptr;
-    strcpy (tname, diqinfo->name);
-    strcat (tname, "/quotas");
+    strncpy (tname, diqinfo->name, DI_NAME_LEN);
+    strncat (tname, "/quotas", DI_NAME_LEN);
     fd = open (tname, O_RDONLY | O_NOCTTY);
     if (fd >= 0) {
       rc = ioctl (fd, Q_QUOTACTL, &qop);
@@ -221,11 +226,11 @@ diquota (diqinfo)
 
 # ifdef GRPQUOTA
   if (rc == 0 || errno != ESRCH) {
-#  if _lib_quotactl && _quotactl_pos == 1
+#  if _lib_quotactl && _quotactl_pos_1
     rc = quotactl (diqinfo->name, gcmd,
           (int) diqinfo->gid, (caddr_t) qiptr);
 #  endif
-#   if _lib_quotactl && _quotactl_pos == 2
+#   if _lib_quotactl && _quotactl_pos_2
     rc = quotactl (gcmd, diqinfo->special,
              (int) diqinfo->gid, (caddr_t) qiptr);
 #  endif
@@ -236,7 +241,7 @@ diquota (diqinfo)
 #endif /* _has_std_quotas */
 }
 
-#if _hdr_rpc_rpc && _hdr_rpcsvc_rquota
+#if _has_std_nfs_quotas
 
 #ifdef RQ_PATHLEN
 # define DI_RQ_PATHLEN  RQ_PATHLEN
@@ -260,7 +265,7 @@ xdr_quota_get (xp, args)
   if (! xdr_string (xp, &args->gqa_pathp, DI_RQ_PATHLEN)) {
     return 0;
   }
-  if (! _gqa_uid_xdr (xp, &args->gqa_uid)) {
+  if (! xdr_gqa_uid (xp, &args->gqa_uid)) {
     return 0;
   }
   return 1;
@@ -302,22 +307,22 @@ xdr_quota_rslt (xp, rslt)
   if (! xdr_bool (xp, &rptr->rq_active)) {
     return 0;
   }
-  if (! _rquota_xdr (xp, &rptr->rq_bhardlimit)) {
+  if (! xdr_rq_bhardlimit (xp, &rptr->rq_bhardlimit)) {
     return 0;
   }
-  if (! _rquota_xdr (xp, &rptr->rq_bsoftlimit)) {
+  if (! xdr_rq_bsoftlimit (xp, &rptr->rq_bsoftlimit)) {
     return 0;
   }
-  if (! _rquota_xdr (xp, &rptr->rq_curblocks)) {
+  if (! xdr_rq_curblocks (xp, &rptr->rq_curblocks)) {
     return 0;
   }
-  if (! _rquota_xdr (xp, &rptr->rq_fhardlimit)) {
+  if (! xdr_rq_fhardlimit (xp, &rptr->rq_fhardlimit)) {
     return 0;
   }
-  if (! _rquota_xdr (xp, &rptr->rq_fsoftlimit)) {
+  if (! xdr_rq_fsoftlimit (xp, &rptr->rq_fsoftlimit)) {
     return 0;
   }
-  if (! _rquota_xdr (xp, &rptr->rq_curfiles)) {
+  if (! xdr_rq_curfiles (xp, &rptr->rq_curfiles)) {
     return 0;
   }
   return (1);
@@ -334,7 +339,7 @@ diquota_nfs (diqinfo)
     CLIENT                  *rqclnt;
     enum clnt_stat          clnt_stat;
     struct timeval          timeout;
-    char                    host [DI_SPEC_NAME_LEN];
+    char                    host [DI_SPEC_NAME_LEN + 1];
     char                    *ptr;
     char                    *path;
     struct getquota_args    args;
@@ -350,7 +355,7 @@ diquota_nfs (diqinfo)
     timeout.tv_sec = 2;
     timeout.tv_usec = 0;
 
-    strcpy (host, diqinfo->special);
+    strncpy (host, diqinfo->special, DI_SPEC_NAME_LEN);
     path = host;
     ptr = strchr (host, ':');
     if (ptr != (char *) NULL) {
@@ -363,7 +368,7 @@ diquota_nfs (diqinfo)
     args.gqa_pathp = path;
     args.gqa_uid = (int) diqinfo->uid;
 
-    rqclnt = clnt_create (host, (unsigned long) RQUOTAPROG, 
+    rqclnt = clnt_create (host, (unsigned long) RQUOTAPROG,
         (unsigned long) RQUOTAVERS, "udp");
     if (rqclnt == (CLIENT *) NULL) {
       if (debug > 2) {
@@ -437,7 +442,7 @@ diquota_nfs (diqinfo)
     }
     clnt_destroy (rqclnt);
 }
-#endif  /* have rpc headers */
+#endif  /* have std nfs quotas */
 
 #if _has_std_quotas
 static void
@@ -456,7 +461,12 @@ di_process_quotas (tag, diqinfo, rc, xfsflag, cqinfo)
   _fs_size_t        quotBlockSize = { DI_QUOT_BLOCK_SIZE };
   _fs_size_t        tsize;
   _fs_size_t        tlimit;
+# if _typ_struct_dqblk
   struct dqblk      *qinfo;
+# endif
+# if _typ_struct_ufs_dqblk
+  struct ufs_dqblk  *qinfo;
+# endif
 # if _typ_fs_disk_quota_t
   fs_disk_quota_t   *xfsqinfo;
 # endif
@@ -464,7 +474,12 @@ di_process_quotas (tag, diqinfo, rc, xfsflag, cqinfo)
   if (debug > 5) {
     printf ("quota: di_process_quotas\n");
   }
+# if _typ_struct_dqblk
   qinfo = (struct dqblk *) cqinfo;
+# endif
+# if _typ_struct_ufs_dqblk
+  qinfo = (struct ufs_dqblk *) cqinfo;
+# endif
   if (xfsflag) {
 # if _typ_fs_disk_quota_t
     xfsqinfo = (fs_disk_quota_t *) cqinfo;
