@@ -1,7 +1,7 @@
 /*
- * $Id: trimchar.c,v 1.7 2010-07-14 01:55:09-07 bll Exp $
- * $Source: /home/bll/DI/di/RCS/trimchar.c,v $
- * Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
+ * $Id: trimchar.c,v 1.9 2011-09-26 17:50:45-07 bll Exp $
+ * $Source: /home/bll/DI/di/C/RCS/trimchar.c,v $
+ * Copyright 1994-2011 Brad Lanam, Walnut Creek, CA
  */
 
 #include "config.h"
@@ -29,18 +29,18 @@ trimChar (str, ch)
     int          ch;
 #endif
 {
-    Size_t       len;
+  int     len;
 
-    len = strlen (str);
-    if (len > 0)
+  len = (int) strlen (str);
+  if (len > 0)
+  {
+    --len;
+  }
+  if (len >= 0)
+  {
+    if (str [len] == ch)
     {
-        --len;
-    }
-    if (len >= 0)
-    {
-        if (str [len] == ch)
-        {
-            str [len] = '\0';
-        }
+      str [len] = '\0';
     }
+  }
 }
