@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# $Id: mkconfig.pl,v 1.86 2010-12-30 17:06:42-08 bll Exp $
+# $Id: mkconfig.pl,v 1.102 2011-10-05 18:29:18-07 bll Exp $
 #
 # Copyright 2006-2010 Brad Lanam Walnut Creek, CA USA
 #
@@ -14,9 +14,10 @@ my $CONFH;
 my $LOG = "mkconfig.log";
 my $TMP = "_tmp_mkconfig";
 my $OPTIONFILE = "options.dat";
+my $VARSFILE = "mkc_none_c.vars";
 my $CACHEFILE = "mkconfig.cache";
-my $VARSFILE = "mkconfig_c.vars";
 my $REQLIB = "mkconfig.reqlibs";
+my $MKC_DIR = "invalid";
 
 my $precc = <<'_HERE_';
 #if defined(__STDC__) || defined(__cplusplus) || defined(c_plusplus)
@@ -41,6 +42,23 @@ my $optionsloaded = 0;
 my %optionshash;
 my $iflevels = '';
 
+my $awkcmd = 'awk';
+foreach my $p (split /[;:]/o, $ENV{'PATH'})
+{
+    if (-x "$p/awk" && $awkcmd == "") {
+        $awkcmd = "$p/awk";
+    }
+    if (-x "$p/nawk" && $awkcmd !~ /mawk$/o && $awkcmd !~ /gawk$/o) {
+        $awkcmd = "$p/nawk";
+    }
+    if (-x "$p/mawk" && $awkcmd !~ /gawk$/o) {
+        $awkcmd = "$p/mawk";
+    }
+    if (-x "$p/gawk") {
+        $awkcmd = "$p/gawk";
+    }
+}
+
 sub
 exitmkconfig
 {
@@ -89,36 +107,49 @@ printyesno
     printyesno_val $name, $val, $tag;
 }
 
+sub
+savevars
+{
+    my ($r_clist) = @_;
+
+    open (MKCV, ">$VARSFILE");
+    foreach my $val (@{$r_clist->{'vars'}})
+    {
+      print MKCV $val, "\n";
+    }
+    close (MKCV);
+}
+
 sub
 savecache
 {
     my ($r_clist, $r_config) = @_;
 
     open (MKCC, ">$CACHEFILE");
-    open (MKCV, ">$VARSFILE");
-    foreach my $val (@{$r_clist->{'list'}})
+    foreach my $val (@{$r_clist->{'clist'}})
     {
       if ($val =~ /^lib__lib_/o) {
         $tval = $val;
         $tval =~ s/^lib_//o;
-        print MKCC "di_c_lib_${tval}='" . $r_config->{$val} . "'\n";
+        print MKCC "mkc_c_lib_${tval}='" . $r_config->{$val} . "'\n";
       } else {
-        print MKCC "di_c_${val}='" . $r_config->{$val} . "'\n";
-        print MKCV $val, "\n";
+        print MKCC "mkc_c_${val}='" . $r_config->{$val} . "'\n";
       }
     }
     close (MKCC);
-    close (MKCV);
 }
 
 sub
 checkcache_val
 {
-  my ($name, $r_config) = @_;
+  my ($name, $r_clist, $r_config) = @_;
 
   my $rc = 1;
   if (defined ($r_config->{$name}) && $r_config->{$name} ne "" )
   {
+    push @{$r_clist->{'vars'}}, $name;
+    my $r_hash = $r_clist->{'vhash'};
+    $r_hash->{$name} = 1;
     printyesno_val $name, $r_config->{$name}, " (cached)";
     $rc = 0;
   }
@@ -128,11 +159,14 @@ checkcache_val
 sub
 checkcache
 {
-  my ($name, $r_config) = @_;
+  my ($name, $r_clist, $r_config) = @_;
 
   my $rc = 1;
   if (defined ($r_config->{$name}) && $r_config->{$name} ne "" )
   {
+    push @{$r_clist->{'vars'}}, $name;
+    my $r_hash = $r_clist->{'vhash'};
+    $r_hash->{$name} = 1;
     printyesno $name, $r_config->{$name}, " (cached)";
     $rc = 0;
   }
@@ -157,16 +191,29 @@ loadoptions
   }
 }
 
+sub
+setclist
+{
+    my ($r_clist, $name) = @_;
+    $r_hash = $r_clist->{'chash'};
+    if (! defined ($r_hash->{$name}))
+    {
+      push @{$r_clist->{'clist'}}, $name;
+      $r_hash->{$name} = 1;
+    }
+}
+
 sub
 setlist
 {
     my ($r_clist, $name) = @_;
-    my $r_hash = $r_clist->{'hash'};
+    my $r_hash = $r_clist->{'vhash'};
     if (! defined ($r_hash->{$name}))
     {
-      push @{$r_clist->{'list'}}, $name;
+      push @{$r_clist->{'vars'}}, $name;
       $r_hash->{$name} = 1;
     }
+    setclist ($r_clist, $name);
 }
 
 sub
@@ -193,7 +240,7 @@ print_headers
 
     if ($r_a->{'incheaders'} eq 'all')
     {
-        foreach my $val (@{$r_clist->{'list'}})
+        foreach my $val (@{$r_clist->{'vars'}})
         {
             if ($val !~ m#^(_hdr_|_sys_)#o)
             {
@@ -314,6 +361,7 @@ _chk_link
       }
     }
 
+    $dlibs =~ s/ *$//o;
     $r_a->{'dlibs'} = $dlibs;
 
     return $rc;
@@ -350,12 +398,14 @@ _chk_link_libs
 }
 
 sub
-_check_cpp
+_chk_cpp
 {
-    my ($name, $code, $r_a, $r_clist, $r_config) = @_;
+    my ($name, $code, $r_clist, $r_config, $r_a) = @_;
 
     open (CCFH, ">$name.c");
     print CCFH $precc;
+    my $hdrs = print_headers ($r_a, $r_clist, $r_config);
+    print CCFH $hdrs;
     print CCFH $code;
     close CCFH;
 
@@ -384,9 +434,7 @@ _chk_compile
     my ($name, $code, $r_clist, $r_config, $r_a) = @_;
 
     open (CCFH, ">$name.c");
-
     print CCFH $precc;
-
     my $hdrs = print_headers ($r_a, $r_clist, $r_config);
     print CCFH $hdrs;
     print CCFH $code;
@@ -425,7 +473,7 @@ check_header
     my ($name, $file, $r_clist, $r_config, $r_a) = @_;
 
     printlabel $name, "header: $file";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -461,7 +509,7 @@ check_constant
     my ($name, $constant, $r_clist, $r_config, $r_a) = @_;
 
     printlabel $name, "constant: $constant";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -487,7 +535,7 @@ check_keyword
     my ($name, $keyword, $r_clist, $r_config) = @_;
 
     printlabel $name, "keyword: $keyword";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -512,7 +560,7 @@ check_proto
     my ($name, $r_clist, $r_config) = @_;
 
     printlabel $name, "supported: prototypes";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -540,7 +588,7 @@ check_command
     my ($name, $cmd, $r_clist, $r_config) = @_;
 
     printlabel $name, "command: $cmd";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -624,8 +672,46 @@ check_if
     printlabel $name, "if ($ifcount): $iflabel";
     my $nline = '';
     print LOGFH "## ifline: $ifline\n";
+    my $ineq = 0;
+    my $val;
+    my $quoted = 0;
+    my $qtoken;
     foreach my $token (split (/\s/, $ifline)) {
-      if ($token eq '(' || $token eq ')' || $token eq '&&' ||
+      if ($token =~ /^'.*'$/o) {
+        print LOGFH "## start/end qtoken\n";
+        $token =~ s/'//go;
+      } elsif ($token =~ /^'/o) {
+        $qtoken = $token;
+        $quoted = 1;
+        print LOGFH "## start qtoken\n";
+        next;
+      }
+      if ($quoted) {
+        if ($token =~ /'$/o) {
+          $token = $qtoken . ' ' . $token;
+          $token =~ s/'//go;
+          print LOGFH "## end qtoken\n";
+          $quoted = 0;
+        } else {
+          $qtoken = $qtoken . ' ' . $token;
+          print LOGFH "## in qtoken\n";
+          next;
+        }
+      }
+
+      if ($ineq == 1) {
+        $ineq = 2;
+        print LOGFH "## value of $token(B): " . $r_config->{$token} . "\n";
+        $val = $r_config->{$token};
+      } elsif ($ineq == 2) {
+        print LOGFH "## end ==\n";
+        $ineq = 0;
+        $nline .= "('$val' eq '$token') ? 1 : 0";
+        $nline .= ' ';
+      } elsif ($token eq '==') {
+        print LOGFH "## begin ==\n";
+        $ineq = 1;
+      } elsif ($token eq '(' || $token eq ')' || $token eq '&&' ||
           $token eq '||' || $token eq '!') {
         $nline .= $token . ' ';
       } else {
@@ -689,202 +775,6 @@ check_option
     printyesno_actual $name, $r_config->{$name};
 }
 
-sub
-check_quotactl_pos
-{
-  my ($name, $r_clist, $r_config) = @_;
-
-  printlabel $name, "quotactl position";
-  if (checkcache ($name, $r_config) == 0) {
-    return;
-  }
-
-  setlist $r_clist, $name;
-
-  if ($r_config->{'_lib_quotactl'} eq '0') {
-    $r_config->{$name} = '0';
-    printyesno_val $name, $r_config->{$name};
-    return;
-  }
-
-  my $hdr = '';
-  foreach my $hnm ('_sys_quota', '_hdr_ufs_ufs_quota', '_hdr_ufs_quota',
-      '_hdr_linux_quota') {
-    if ($r_config->{$hnm} ne '0') {
-      $hdr = $r_config->{$hnm};
-      last;
-    }
-  }
-  my $uhdr = $r_config->{'_hdr_unistd'};
-
-  my $code = '';
-  if ($hdr ne '') {
-    $code .= "#include <${hdr}>\n";
-    if ($uhdr ne '0') {
-      $code .= "#include <${uhdr}>\n";
-    }
-    my $rc = _check_cpp ($name, $code, {}, $r_clist, $r_config);
-    if ($rc == 0) {
-       $cmd = "egrep -l 'quotactl *\\( *int[^,]*, *(const *)?char' $name.out >>$LOG 2>&1";
-       print LOGFH "##  quotactl_pos: $cmd\n";
-       $rc = system ($cmd);
-       if ($rc == 0) {
-         $r_config->{$name} = 2;
-         printyesno_val $name, $r_config->{$name};
-         return;
-       }
-
-       $cmd = "egrep -l 'quotactl *\\( *(const *)?char[^,]*, *int' $name.out >>$LOG 2>&1";
-       print LOGFH "##  quotactl_pos: $cmd\n";
-       $rc = system ($cmd);
-       if ($rc == 0) {
-         $r_config->{$name} = 1;
-         printyesno_val $name, $r_config->{$name};
-         return;
-       }
-    }
-  }
-
-  $r_config->{$name} = 0;
-  printyesno_val $name, $r_config->{$name};
-}
-
-sub
-check_rquota_xdr
-{
-  my ($name, $r_clist, $r_config) = @_;
-
-  printlabel $name, "rquota xdr";
-  if (checkcache ($name, $r_config) == 0) {
-    return;
-  }
-
-  setlist $r_clist, $name;
-
-  if ($r_config->{'_hdr_rpcsvc_rquota'} eq '0') {
-    $r_config->{$name} = '0';
-    printyesno_val $name, $r_config->{$name};
-    return;
-  }
-
-  my $code = '';
-  my $hdr = $r_config->{'_hdr_rpcsvc_rquota'};
-  $code .= "#include <$hdr>\n";
-  my $rc = _check_cpp ($name, $code, {}, $r_clist, $r_config);
-  if ($rc == 0) {
-    $cmd = "sed '1,/struct *rquota *{/d' $name.out |
-        egrep '[	 ]*rq_bhardlimit[	 ]*;' >>$LOG 2>&1";
-    print LOGFH "##  rquota_xdr: $cmd\n";
-    $rc = system ($cmd);
-    if ($rc == 0) {
-      my $rval = `sed '1,/struct *rquota *{/d' $name.out |
-          egrep '[	 ]*rq_bhardlimit[	 ]*;'`;
-      chomp $rval;
-      $rval =~ s/[	 ][	 ]*rq_bhardlimit.*//;
-      $rval =~ s/^[	 ]*//;
-      $r_config->{$name} = "xdr_" . ${rval};
-      printyesno_val $name, $r_config->{$name};
-      return;
-    }
-  }
-
-  $r_config->{$name} = 0;
-  printyesno_val $name, $r_config->{$name};
-}
-
-sub
-check_gqa_uid_xdr
-{
-  my ($name, $r_clist, $r_config) = @_;
-
-  printlabel $name, "gqa_uid xdr";
-  if (checkcache ($name, $r_config) == 0) {
-    return;
-  }
-
-  setlist $r_clist, $name;
-
-  if ($r_config->{'_hdr_rpcsvc_rquota'} eq '0') {
-    $r_config->{$name} = '0';
-    printyesno_val $name, $r_config->{$name};
-    return;
-  }
-
-  my $code = '';
-  my $hdr = $r_config->{'_hdr_rpcsvc_rquota'};
-  $code .= "#include <$hdr>\n";
-  my $rc = _check_cpp ($name, $code, {}, $r_clist, $r_config);
-  if ($rc == 0) {
-    $cmd = "sed '1,/struct *getquota_args *{/d' $name.out |
-        egrep '[	 ]*gqa_uid[	 ]*;' >>$LOG 2>&1";
-    print LOGFH "##  gqa_uid_xdr: $cmd\n";
-    $rc = system ($cmd);
-    if ($rc == 0) {
-      my $rval = `sed '1,/struct *getquota_args *{/d' $name.out |
-          egrep '[	 ]*gqa_uid[	 ]*;'`;
-      chomp $rval;
-      $rval =~ s/[	 ][	 ]*gqa_uid.*//;
-      $rval =~ s/^[	 ]*//;
-      $r_config->{$name} = "xdr_" . ${rval};
-      printyesno_val $name, $r_config->{$name};
-      return;
-    }
-  }
-
-  $r_config->{$name} = 0;
-  printyesno_val $name, $r_config->{$name};
-}
-
-sub
-check_getfsstat_type
-{
-  my ($name, $r_clist, $r_config) = @_;
-
-  printlabel $name, "getfsstat type";
-  if (checkcache ($name, $r_config) == 0) {
-    return;
-  }
-
-  setlist $r_clist, $name;
-
-  if ($r_config->{'_lib_getfsstat'} eq '0') {
-    $r_config->{$name} = '0';
-    printyesno_val $name, $r_config->{$name};
-    return;
-  }
-
-  if ($r_config->{'_sys_mount'} eq '0') {
-    $r_config->{$name} = '0';
-    printyesno_val $name, $r_config->{$name};
-    return;
-  }
-
-  my $code = '';
-  my $hdr = $r_config->{'_sys_mount'};
-  $code .= "#include <$hdr>\n";
-  my $rc = _check_cpp ($name, $code, {}, $r_clist, $r_config);
-  if ($rc == 0) {
-    $cmd = "egrep -l 'getfsstat[	 ]*\\(' $name.out >>$LOG 2>&1";
-    $rc = system ($cmd);
-    print LOGFH "##  getfsstat_type: $cmd $rc\n";
-    if ($rc == 0) {
-      my $rval = `egrep 'getfsstat[	 ]*\\(' $name.out`;
-      print LOGFH "##  getfsstat_type: $rval\n";
-      chomp $rval;
-      $rval =~ s/^[^,]*,[	 ]*//;
-      $rval =~ s/,.*$//;
-      $rval =~ s/  *[_a-z0-9]*$//;  # strip any trailing var name
-      print LOGFH "##  getfsstat_type: $rval\n";
-      $r_config->{$name} = ${rval};
-      printyesno_val $name, $r_config->{$name};
-      return;
-    }
-  }
-
-  $r_config->{$name} = 0;
-  printyesno_val $name, $r_config->{$name};
-}
-
 sub
 check_include_conflict
 {
@@ -917,7 +807,7 @@ check_include_conflict
         defined ($r_config->{${i2}}) &&
         $r_config->{${i2}} ne '0')
     {
-        if (checkcache ($name, $r_config) == 0)
+        if (checkcache ($name, $r_clist, $r_config) == 0)
         {
             return;
         }
@@ -941,7 +831,7 @@ check_npt
     my ($name, $proto, $req, $r_clist, $r_config) = @_;
 
     printlabel $name, "need prototype: $proto";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -970,7 +860,7 @@ check_type
 
     $type =~ s/star/*/og;
     printlabel $name, "type: $type";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -990,7 +880,7 @@ check_defined
     my ($name, $def, $r_clist, $r_config) = @_;
 
     printlabel $name, "defined: $def";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -1017,7 +907,7 @@ check_param_void_star
     my ($name, $r_clist, $r_config) = @_;
 
     printlabel $name, "parameter: void *";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -1051,7 +941,7 @@ check_lib
     else
     {
         printlabel $name, "function: $rfunc";
-        if (checkcache ($name, $r_config) == 0)
+        if (checkcache ($name, $r_clist, $r_config) == 0)
         {
             return;
         }
@@ -1079,7 +969,7 @@ _HERE_
       {
           $tag = " with $a{'dlibs'}";
           $r_config->{"lib_$name"} = $a{'dlibs'};
-          setlist $r_clist, "lib_$name";
+          setclist $r_clist, "lib_$name";
       }
     }
     printyesno $name, $r_config->{$name}, $tag;
@@ -1100,7 +990,7 @@ check_class
     else
     {
         printlabel $name, "class: $class";
-        if (checkcache ($name, $r_config) == 0)
+        if (checkcache ($name, $r_clist, $r_config) == 0)
         {
             return;
         }
@@ -1131,111 +1021,105 @@ _HERE_
 }
 
 sub
-check_setmntent_args
+check_args
 {
-    my ($name, $r_clist, $r_config) = @_;
+    my ($name, $funcnm, $r_a, $r_clist, $r_config) = @_;
 
-    printlabel $name, "setmntent # arguments";
-    if (checkcache ($name, $r_config) == 0)
+    printlabel $name, "args: $funcnm";
+    if (checkcache_val ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
 
-    setlist $r_clist, $name;
-    $r_config->{$name} = 0;
-
-    if ($r_config->{'_lib_setmntent'} eq '0')
-    {
-        printyesno_val $name, $r_config->{$name};
-        return;
-    }
-    my $code = <<"_HERE_";
-main () { setmntent ("/etc/mnttab"); }
-_HERE_
-    my $rc = _chk_link ($name, $code, $r_clist, $r_config,
-        { 'incheaders' => 'all', 'otherlibs' => undef, });
-    if ($rc == 0)
-    {
-        $r_config->{$name} = 2;
-        printyesno_val $name, $r_config->{$name};
+    if ($ENV{'_MKCONFIG_USING_GCC'} == 'N' &&
+        $ENV{'_MKCONFIG_SYSTYPE'} == 'HP-UX' ) {
+      my $tcc = `${CC} -v 2>&1`;
+      if ($tcc =~ /Bundled/o) {
+        print " bundled cc; skipped";
         return;
+      }
     }
 
-    $code = <<"_HERE_";
-main () { setmntent ("/etc/mnttab", "r"); }
-_HERE_
-    $rc = _chk_link ($name, $code, $r_clist, $r_config,
-        { 'incheaders' => 'all', 'otherlibs' => undef, });
-    if ($rc == 0)
-    {
-        $r_config->{$name} = 3;
-    }
-    printyesno_val $name, $r_config->{$name};
-}
+    my $oldprecc = $precc;
+    $precc .= "/* get rid of most gcc-isms */
+#define __asm__(a)
+#define __attribute__(a)
+#define __nonnull__(a,b)
+#define __restrict
+#define __THROW
+#define __const const
+";
 
-sub
-check_statfs_args
-{
-    my ($name, $r_clist, $r_config) = @_;
+   my $rc = _chk_cpp ($name, "", $r_clist, $r_config,
+        { 'incheaders' => 'all', });
+   $precc = $oldprecc;
+   my $ccount = 0;
 
-    printlabel $name, "statfs # arguments";
-    if (checkcache ($name, $r_config) == 0)
-    {
-        return;
-    }
+   if ($rc == 0) {
+     my $cmd = "egrep \"[	 *]" . ${funcnm} . "[	 ]*\\(\" $name.out >/dev/null 2>&1";
+     $rc = system ($cmd);
+     print LOGFH "##  args: $cmd $rc\n";
+     if ($rc == 0) {
+       my $dcl = `${awkcmd} -f ${MKC_DIR}/mkcextdcl.awk ${name}.out ${funcnm}`;
+       # $dcl may be multi-line...fix this now.
+       $dcl =~ s/[ 	\n]/ /gos;
+       $dcl =~ s/extern *//o;
+       $dcl =~ s/;//o;
+       print LOGFH "##  dcl(A): $dcl\n";
+       $dcl =~ s/\( *void *\)/()/o;
+       print LOGFH "##  dcl(C): $dcl\n";
 
-    setlist $r_clist, $name;
-    $r_config->{$name} = 0;
+       my $c = $dcl;
+       $c =~ s/[^,]*//go;
+       $ccount = length ($c);
+       $ccount += 1;
 
-    if ($r_config->{'_lib_statfs'} eq '0')
-    {
-        printyesno_val $name, $r_config->{$name};
-        return;
-    }
+       $c = $dcl;
+       $c =~ s/^[^\(]*\(//o;
+       $c =~ s/\)[^\)]*$//o;
+       print LOGFH "## c(E): ${c}\n";
+       my $val = 1;
+       while (${c} ne "") {
+         my $tc = $c;
+         $tc =~ s/ *,.*$//o;
+         $tc =~ s/[ 	]+/ /go;
+         if ($r_a->{'noconst'} eq 'T') {
+           $tc =~ s/const *//o;
+         }
+         $tc =~ s/^ *//;
+         $tc =~ s/ *$//;
+         # only do the following if the names of the variables are declared
+         $tc =~ s/(struct|union|enum) /$1#/;
+         if ($tc =~ / /o) {
+           $tc =~ s/ *[A-Za-z0-9_]*$//o;
+         }
+         $tc =~ s/(struct|union|enum)#/$1 /;
+         print LOGFH "## tc(F): ${tc}\n";
+         my $nm = "_c_arg_${val}_${funcnm}";
+         setlist $r_clist, $nm;
+         $r_config->{$nm} = $tc;
+         $val += 1;
+         $c =~ s/^[^,]*//o;
+         $c =~ s/^[	 ,]*//o;
+         print LOGFH "## c(G): ${c}\n";
+       }
+       $c = $dcl;
+       $c =~ s/[ 	]+/ /go;
+       $c =~ s/([ \*])${funcnm}[ \(].*/$1/;
+       $c =~ s/^ *//o;
+       $c =~ s/ *$//o;
+       if ($r_a->{'noconst'}) {
+         $c =~ s/const *//;
+       }
+       $nm = "_c_type_${funcnm}";
+       setlist $r_clist, $nm;
+       $r_config->{$nm} = $c;
+     }
+   }
 
-    my $code = <<"_HERE_";
-main () {
-    struct statfs statBuf; char *name; name = "/";
-    statfs (name, &statBuf);
-}
-_HERE_
-    my $rc = _chk_link ($name, $code, $r_clist, $r_config,
-        { 'incheaders' => 'all', 'otherlibs' => undef, });
-    if ($rc == 0)
-    {
-        $r_config->{$name} = 2;
-        printyesno_val $name, $r_config->{$name};
-        return;
-    }
-
-    $code = <<"_HERE_";
-main () {
-    struct statfs statBuf; char *name; name = "/";
-    statfs (name, &statBuf, sizeof (statBuf));
-}
-_HERE_
-    $rc = _chk_link ($name, $code, $r_clist, $r_config,
-        { 'incheaders' => 'all', 'otherlibs' => undef, });
-    if ($rc == 0)
-    {
-        $r_config->{$name} = 3;
-        printyesno_val $name, $r_config->{$name};
-        return;
-    }
-
-    $code = <<"_HERE_";
-main () {
-    struct statfs statBuf; char *name; name = "/";
-    statfs (name, &statBuf, sizeof (statBuf), 0);
-}
-_HERE_
-    $rc = _chk_link ($name, $code, $r_clist, $r_config,
-        { 'incheaders' => 'all', 'otherlibs' => undef, });
-    if ($rc == 0)
-    {
-        $r_config->{$name} = 4;
-    }
-    printyesno_val $name, $r_config->{$name};
+   setlist $r_clist, $name;
+   $r_config->{$name} = $ccount;
+   printyesno_val $name, $r_config->{$name};
 }
 
 sub
@@ -1244,7 +1128,7 @@ check_size
     my ($name, $type, $r_clist, $r_config) = @_;
 
     printlabel $name, "sizeof: $type";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -1272,7 +1156,7 @@ check_member
     my ($name, $struct, $member, $r_clist, $r_config) = @_;
 
     printlabel $name, "exists: $struct.$member";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -1291,13 +1175,46 @@ _HERE_
     printyesno $name, $r_config->{$name};
 }
 
+sub
+check_memberxdr
+{
+    my ($name, $struct, $member, $r_clist, $r_config) = @_;
+
+    printlabel $name, "member:xdr: $struct.$member";
+    if (checkcache ($name, $r_clist, $r_config) == 0)
+    {
+        return;
+    }
+
+    $r_config->{$name} = 0;
+    my $rc = _chk_cpp ($name, "", $r_clist, $r_config,
+        { 'incheaders' => 'all', });
+    if ($rc == 0) {
+      print LOGFH `pwd` . "\n";
+      print LOGFH "## ${awkcmd} -f ${MKC_DIR}/mkcextstruct.awk ${name}.out ${struct}\n";
+      my $st = `${awkcmd} -f ${MKC_DIR}/mkcextstruct.awk ${name}.out ${struct}`;
+      print LOGFH "##  xdr(A): $st\n";
+      if ($st =~ m/${member}\s*;/s) {
+        my $mtype = $st;
+        $mtype =~ s/\s*${member}\s*;.*//s;
+        $mtype =~ s/.*\s//os;
+        print LOGFH "##  xdr(B): $mtype\n";
+        setlist $r_clist, "xdr_${member}";
+        $r_config->{"xdr_${member}"} = "xdr_${mtype}";
+        $r_config->{$name} = 1;
+      }
+    }
+    setlist $r_clist, $name;
+    printyesno $name, $r_config->{$name};
+}
+
 sub
 check_int_declare
 {
     my ($name, $function, $r_clist, $r_config) = @_;
 
     printlabel $name, "declared: $function";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -1322,7 +1239,7 @@ check_ptr_declare
     my ($name, $function, $r_clist, $r_config) = @_;
 
     printlabel $name, "declared: $function";
-    if (checkcache ($name, $r_config) == 0)
+    if (checkcache ($name, $r_clist, $r_config) == 0)
     {
         return;
     }
@@ -1374,13 +1291,13 @@ create_output
   if ($CONFH eq 'none') { return; }
 
   open (CCOFH, ">$CONFH");
-  print CCOFH <<'_HERE_';
-#ifndef __INC_CONFIG_H
-#define __INC_CONFIG_H 1
+  print CCOFH <<"_HERE_";
+#ifndef __INC_${CONFHTAGUC}_H
+#define __INC_${CONFHTAGUC}_H 1
 
 _HERE_
 
-  foreach my $val (@{$r_clist->{'list'}})
+  foreach my $val (@{$r_clist->{'vars'}})
   {
     if ($val =~ m#^lib__lib_#o) {
       next;
@@ -1393,7 +1310,7 @@ _HERE_
       $tnm = $val;
       $tnm =~ s/^_setint_//;
       print CCOFH "#define $tnm " . $r_config->{$val} . "\n";
-    } elsif ($val =~ m#^_setstr_#o || $val =~ m#^_opt_#o) {
+    } elsif ($val =~ m#^(_setstr_|_opt_)#o) {
       $tnm = $val;
       $tnm =~ s/^_setstr_//;
       $tnm =~ s/^_opt_//;
@@ -1432,9 +1349,9 @@ _HERE_
 
   print CCOFH $include;
 
-  print CCOFH <<'_HERE_';
+  print CCOFH <<"_HERE_";
 
-#endif /* __INC_CONFIG_H */
+#endif /* __INC_${CONFHTAGUC}_H */
 _HERE_
   close CCOFH;
 }
@@ -1445,36 +1362,28 @@ main_process
     my ($configfile) = @_;
     my (%clist, %config);
 
-    $clist{'list'} = ();
-    $clist{'hash'} = {};
+    $clist{'clist'} = ();
+    $clist{'chash'} = {};
+    $clist{'vars'} = ();
+    $clist{'vhash'} = {};
     $config{'reqlibs'} = {};
     $config{'reqlibs_list'} = ();
 
-    if (-f $CACHEFILE)
-    {
+    if (-f $CACHEFILE) {
       open (MKCC, "<$CACHEFILE");
-      while (my $line = <MKCC>)
-      {
+      while (my $line = <MKCC>) {
         chomp $line;
-        if ($line =~ m/^di_c_(.*)='(.*)'/o)
-        {
+        if ($line =~ m/^mkc_c_(.*)=\$?'?(.*)'?/o) {
           my $name = $1;
           my $val = $2;
           $config{$name} = $val;
-          $clist{'hash'}->{$name} = 1;
+          push @{$clist{'clist'}}, $name;
+          $clist{'chash'}->{$name} = 1;
         }
       }
       close (MKCC);
-      open (MKCV, "<$VARSFILE");
-      while (my $line = <MKCV>)
-      {
-        chomp $line;
-        push @{$clist{'list'}}, $line;
-      }
-      close (MKCV);
     }
 
-
     my $tconfigfile = $configfile;
     if ($configfile !~ m#^/#) {
       $tconfigfile = "../$configfile";
@@ -1558,8 +1467,11 @@ main_process
         if ($line =~ m#^\s*output\s+([^\s]+)#o)
         {
             if ($inproc == 1) {
+              savevars (\%clist);
               create_output (\%clist, \%config, $include);
               $CONFH = 'none';
+              $CONFHTAG = 'none';
+              $CONFHTAGUC = 'NONE';
               $include = '';
             }
 
@@ -1570,8 +1482,15 @@ main_process
             } else {
               $CONFH = "../$tconfh";
             }
+            $CONFHTAG = $tconfh;
+            $CONFHTAG =~ s,.*/,,;
+            $CONFHTAG =~ s,\..*$,,;
+            $CONFHTAGUC = uc $CONFHTAG;
             print LOGFH "config file: $CONFH\n";
             $inproc = 1;
+            $VARSFILE = "../mkc_${CONFHTAG}_c.vars";
+            $clist{'vars'} = ();
+            $clist{'vhash'} = {};
         }
         elsif ($line =~ m#^\s*option\-file\s+([^\s]+)#o)
         {
@@ -1592,13 +1511,20 @@ main_process
         {
             ;
         }
-        elsif ($line =~ m#^\s*setmntent_args#o)
+        elsif ($line =~ m#^\s*args\s+(.*)$#o)
         {
-            check_setmntent_args ('_setmntent_args', \%clist, \%config);
-        }
-        elsif ($line =~ m#^\s*statfs_args#o)
-        {
-            check_statfs_args ('_statfs_args', \%clist, \%config);
+            my $funcnm = $1;
+
+            %args = ( 'noconst' => 'F' );
+            if ($funcnm =~ /^noconst */o) {
+              $funcnm =~ s/^noconst *//o;
+              $args{'noconst'} = 'T';
+            }
+
+            my $nm = "_args_${funcnm}";
+            $nm =~ s,/,_,go;
+            $nm =~ s,:,_,go;
+            check_args ($nm, $funcnm, \%args, \%clist, \%config);
         }
         elsif ($line =~ m#^\s*include_conflict\s+([^\s]+)\s+([^\s]+)#o)
         {
@@ -1606,10 +1532,6 @@ main_process
             my $i2 = $2;
             check_include_conflict ('_inc_conflict', $i1, $i2, \%clist, \%config);
         }
-        elsif ($line =~ m#^\s*quotactl_pos$#o)
-        {
-            check_quotactl_pos ('_quotactl_pos', \%clist, \%config);
-        }
         elsif ($line =~ m#^\s*rquota_xdr$#o)
         {
             check_rquota_xdr ('_rquota_xdr', \%clist, \%config);
@@ -1618,10 +1540,6 @@ main_process
         {
             check_gqa_uid_xdr ('_gqa_uid_xdr', \%clist, \%config);
         }
-        elsif ($line =~ m#^\s*getfsstat_type$#o)
-        {
-            check_getfsstat_type ('_getfsstat_type', \%clist, \%config);
-        }
         elsif ($line =~ m#^\s*include$#o)
         {
             print LOGFH "start include\n";
@@ -1693,6 +1611,14 @@ main_process
         {
             ;
         }
+        elsif ($line =~ m#^\s*echo\s+(.*)$#o)
+        {
+            print $1;
+        }
+        elsif ($line =~ m#^\s*exit$#o)
+        {
+            print $1;
+        }
         elsif ($line =~ m#^\s*(set(int|str)?)\s+([^\s]+)\s*(.*)#o)
         {
             my $type = $1;
@@ -1803,6 +1729,18 @@ main_process
                 check_member ($nm, $struct, $member, \%clist, \%config);
             }
         }
+        elsif ($line =~ m#^\s*memberxdr\s+(.*)\s+([^\s]+)#o)
+        {
+            my $struct = $1;
+            my $member = $2;
+            my $nm = "_memberxdr_" . $struct . '_' . $member;
+            $nm =~ s/ /_/go;
+            if (! defined ($config{$nm}) ||
+                $config{$nm} eq '0')
+            {
+                check_memberxdr ($nm, $struct, $member, \%clist, \%config);
+            }
+        }
         elsif ($line =~ m#^\s*size\s+(.*)#o)
         {
             my $typ = $1;
@@ -1823,6 +1761,7 @@ main_process
       }
     }
 
+    savevars (\%clist);
     savecache (\%clist, \%config);
     create_output (\%clist, \%config, $include);
 
@@ -1895,6 +1834,24 @@ $CACHEFILE = "../$CACHEFILE";
 $VARSFILE = "../$VARSFILE";
 $OPTIONFILE = "../$OPTIONFILE";
 
+delete $ENV{'CDPATH'};
+delete $ENV{'GREP_OPTIONS'};
+delete $ENV{'ENV'};
+$ENV{'LC_ALL'} = "C";
+
+my $tpath = $0;
+$tpath =~ s,/[^/]*$,,;
+my $currdir =`pwd`;
+chomp $currdir;
+if (! chdir $tpath) {
+  die ("Unable to cd to $tpath. $!\n");
+}
+$MKC_DIR = `pwd`;
+chomp $MKC_DIR;
+if (! chdir $currdir) {
+  die ("Unable to cd to $currdir. $!\n");
+}
+
 if (-d $TMP) { system ("rm -rf $TMP"); }
 mkdir $TMP, 0777;
 chdir $TMP;
@@ -1914,11 +1871,14 @@ print LOGFH "CFLAGS: $ENV{'CFLAGS'}\n";
 print LOGFH "CPPFLAGS: $ENV{'CPPFLAGS'}\n";
 print LOGFH "LDFLAGS: $ENV{'LDFLAGS'}\n";
 print LOGFH "LIBS: $ENV{'LIBS'}\n";
+print LOGFH "awk: $awkcmd\n";
 
 main_process $configfile;
 
 close LOGFH;
 
 chdir "..";
-if (-d $TMP) { system ("rm -rf $TMP"); }
+if ($ENV{'MKC_KEEP_TMP'} eq "") {
+  if (-d $TMP) { system ("rm -rf $TMP"); }
+}
 exit 0;
