@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# $Id: mkconfig.sh,v 1.84 2010-12-28 20:45:52-08 bll Exp $
+# $Id: mkconfig.sh,v 1.92 2011-10-05 16:27:26-07 bll Exp $
 #
 # Copyright 2009-2010 Brad Lanam Walnut Creek, CA USA
 #
@@ -8,13 +8,15 @@
 #
 # speed at the cost of maintainability...
 # File Descriptors:
-#    9 - >>$LOG                     (mkconfig.sh)
-#    8 - >>$VARSFILE, >>$CONFH      (mkconfig.sh)
-#    7 - saved stdin                (mkconfig.sh)
-#    6 - temporary                  (c-main.sh, mkconfig.sh)
-#    4 - temporary                  (c-main.sh)
+#    9 - >>$LOG                 (mkconfig.sh)
+#    8 - >>$VARSFILE, >>$CONFH  (mkconfig.sh)
+#    7 - saved stdin            (mkconfig.sh)
+#    6 - temporary              (c-main.sh, mkconfig.sh)
+#    4 - temporary              (c-main.sh)
 #
 
+set -f  # set this globally.
+
 # this is a workaround for ksh93 on solaris
 if [ "$1" = "-d" ]; then
   cd $2
@@ -34,6 +36,7 @@ LOG="mkconfig.log"
 _MKCONFIG_TMP="_tmp_mkconfig"
 CACHEFILE="mkconfig.cache"
 OPTIONFILE="options.dat"
+_MKCONFIG_PREFIX=mkc    # need a default in case no units loaded
 optionsloaded=F
 
 INC="mkcinclude.txt"                   # temporary
@@ -47,7 +50,7 @@ _chkconfigfname () {
 
 _exitmkconfig () {
     rc=$1
-    exit 1
+    exit $rc
 }
 
 _savecache () {
@@ -57,35 +60,52 @@ _savecache () {
     # Then we have to undo it for bash.
     # Other shells do: x=$''; remove the $
     # And then there's: x='', which gets munged.
-    set | grep "^di_" | \
+    set | grep "^mkc_" | \
       sed -e "s/=/='/" -e "s/$/'/" -e "s/''/'/g" \
       -e "s/='$/=''/" -e "s/='\$'/='/" \
       > ${CACHEFILE}
 }
 
+setvar () {
+    prefix=$1
+    svname=$2
+
+    if [ $varsfileopen = F ]; then
+      for v in `set | grep "^mkv_" | sed 's/=.*$//'`; do
+        unset $v
+      done
+      varsfileopen=T
+      exec 8>>$VARSFILE
+    fi
+
+    cmd="test \"X\$mkv_${prefix}_${svname}\" != X > /dev/null 2>&1"
+    eval $cmd
+    rc=$?
+    # if already in the list of vars, don't add it to the file again.
+    if [ $rc -ne 0 ]; then
+      if [ $rc -ne 0 ]; then
+        echo ${svname} >&8
+      fi
+    fi
+
+    cmd="mkv_${prefix}_${svname}=T"
+    eval $cmd
+    echo "   setvar: $cmd" >&9
+}
+
 setdata () {
     prefix=$1
     sdname=$2
     sdval=$3
 
-    cmd="test \"X\$di_${prefix}_${sdname}\" != X > /dev/null 2>&1"
-    eval $cmd
-    rc=$?
-    # if already in the list of vars, don't add it again.
-    if [ $rc -ne 0 ]; then
-      if [ "$_MKCONFIG_HASEMPTY" = "T" ]; then
-        # have to check again, as empty vars don't work for the above test.
-        # need a better way to do this.
-        grep -l "^${sdname}\$" $VARSFILE > /dev/null 2>&1
-        rc=$?
-      fi
-      if [ $rc -ne 0 ]; then
-        echo ${sdname} >&8
-      fi
+    if [ "$_MKCONFIG_EXPORT" = T ]; then
+      _doexport $sdname "$sdval"
     fi
-    cmd="di_${prefix}_${sdname}=\"${sdval}\""
+
+    cmd="mkc_${prefix}_${sdname}=\"${sdval}\""
     eval $cmd
-    echo "   set: $cmd" >&9
+    echo "   setdata: $cmd" >&9
+    setvar $prefix $sdname
 }
 
 getdata () {
@@ -93,7 +113,7 @@ getdata () {
     prefix=$2
     gdname=$3
 
-    cmd="${var}=\${di_${prefix}_${gdname}}"
+    cmd="${var}=\${mkc_${prefix}_${gdname}}"
     eval $cmd
 }
 
@@ -132,10 +152,6 @@ printyesno_actual () {
 
   echo "   [${ynname}] $ynval ${yntag}" >&9
   echo "$ynval ${yntag}" >&1
-
-  if [ "$_MKCONFIG_EXPORT" = "T" ]; then
-    _doexport $ynname "$ynval"
-  fi
 }
 
 printyesno_val () {
@@ -143,7 +159,7 @@ printyesno_val () {
   ynval=$2
   yntag=${3:-}
 
-  if [ "$ynval" != "0" ]; then
+  if [ "$ynval" != 0 ]; then
     printyesno_actual "$ynname" "$ynval" "${yntag}"
   else
     printyesno_actual "$ynname" no "${yntag}"
@@ -155,25 +171,12 @@ printyesno () {
     ynval=$2
     yntag=${3:-}
 
-    if [ "$ynval" != "0" ]; then
+    if [ "$ynval" != 0 ]; then
       ynval=yes
     fi
     printyesno_val "$ynname" $ynval "$yntag"
 }
 
-checkcache_actual () {
-  prefix=$1
-  tname=$2
-
-  getdata tval ${prefix} ${tname}
-  rc=1
-  if [ "$tval" != "" ]; then
-    printyesno_actual $tname "$tval" " (cached)"
-    rc=0
-  fi
-  return $rc
-}
-
 checkcache_val () {
   prefix=$1
   tname=$2
@@ -181,7 +184,8 @@ checkcache_val () {
   getdata tval ${prefix} ${tname}
   rc=1
   if [ "$tval" != "" ]; then
-    printyesno_val $tname "$tval" " (cached)"
+    setvar $prefix $tname
+    printyesno_actual $tname "$tval" " (cached)"
     rc=0
   fi
   return $rc
@@ -194,6 +198,7 @@ checkcache () {
   getdata tval ${prefix} ${tname}
   rc=1
   if [ "$tval" != "" ]; then
+    setvar $prefix $tname
     printyesno $tname $tval " (cached)"
     rc=0
   fi
@@ -201,7 +206,7 @@ checkcache () {
 }
 
 _loadoptions () {
-  if [ $optionsloaded = "F" -a -f "${OPTIONFILE}" ]; then
+  if [ $optionsloaded = F -a -f "${OPTIONFILE}" ]; then
     exec 6<&0 < ${OPTIONFILE}
     while read o; do
       case $o in
@@ -248,32 +253,32 @@ check_ifoption () {
     trc=F  # if option is not set, it's false
 
     found=F
-    if [ "$optionsloaded" = "T" ]; then
+    if [ "$optionsloaded" = T ]; then
       eval tval=\$_mkc_opt_${oopt}
       if [ "$tval" != "" ]; then
         found=T
         trc=$tval
         tolower trc
         echo "  found: $oopt $trc" >&9
-        if [ "$trc" = "t" ]; then trc=T; fi
-        if [ "$trc" = "enable" ]; then trc=T; fi
-        if [ "$trc" = "f" ]; then trc=F; fi
-        if [ "$trc" = "disable" ]; then trc=F; fi
-        if [ "$trc" = "true" ]; then trc=T; fi
-        if [ "$trc" = "false" ]; then trc=F; fi
+        if [ "$trc" = t ]; then trc=T; fi
+        if [ "$trc" = enable ]; then trc=T; fi
+        if [ "$trc" = f ]; then trc=F; fi
+        if [ "$trc" = disable ]; then trc=F; fi
+        if [ "$trc" = true ]; then trc=T; fi
+        if [ "$trc" = false ]; then trc=F; fi
       fi
     fi
 
-    if [ "$trc" = "T" ]; then trc=1; fi
-    if [ "$trc" = "F" ]; then trc=0; fi
+    if [ "$trc" = T ]; then trc=1; fi
+    if [ "$trc" = F ]; then trc=0; fi
 
     if [ $type = "ifnotoption" ]; then
       if [ $trc -eq 0 ]; then trc=1; else trc=0; fi
     fi
-    if [ "$optionsloaded" = "F" ]; then
+    if [ "$optionsloaded" = F ]; then
       trc=0
       printyesno_actual $name "no options file"
-    elif [ "$found" = "F" ]; then
+    elif [ "$found" = F ]; then
       trc=0
       printyesno_actual $name "option not found"
     else
@@ -296,18 +301,63 @@ check_if () {
     trc=0  # if option is not set, it's false
 
     nline="test "
+    ineq=0
+    qtoken=""
+    quoted=0
     for token in $ifline; do
+      echo "## token: $token" >&9
+
       case $token in
-        \(|\)|-a|-o|!)
-          doappend nline " $token"
+        \'*\')
+          token=`echo $token | sed -e s,\',,g`
+          echo "## begin/end quoted token" >&9
+          ;;
+        \'*)
+          qtoken=$token
+          echo "## begin qtoken: $qtoken" >&9
+          quoted=1
+          continue
           ;;
-        *)
-          getdata tvar ${_MKCONFIG_PREFIX} $token
-          if [ "$tvar" != "0" ]; then tvar=1; fi
-          tvar="( $tvar = 1 )"
-          doappend nline " $tvar"
-        ;;
       esac
+
+      if [ $quoted -eq 1 ]; then
+        case $token in
+          *\')
+            token="${qtoken} $token"
+            token=`echo $token | sed -e s,\',,g`
+            echo "## end qtoken: $token" >&9
+            quoted=0
+            ;;
+          *)
+            qtoken="$qtoken $token"
+            echo "## in qtoken: $qtoken" >&9
+            continue
+            ;;
+        esac
+      fi
+
+      if [ $ineq -eq 1 ]; then
+        ineq=2
+        getdata tvar ${_MKCONFIG_PREFIX} $token
+      elif [ $ineq -eq 2 ]; then
+        doappend nline " ( '$tvar' = '$token' )"
+        ineq=0
+      else
+        case $token in
+          ==)
+            ineq=1
+            ;;
+          \(|\)|-a|-o|!)
+            doappend nline " $token"
+            ;;
+          *)
+            getdata tvar ${_MKCONFIG_PREFIX} $token
+            if [ "$tvar" != 0 ]; then tvar=1; fi
+            tvar="( $tvar = 1 )"
+            doappend nline " $tvar"
+          ;;
+        esac
+      fi
     done
 
     if [ "$ifline" != "" ]; then
@@ -367,7 +417,7 @@ check_option () {
   oval=$def
   printlabel $name "option: ${onm}"
 
-  if [ "$optionsloaded" = "T" ]; then
+  if [ "$optionsloaded" = T ]; then
     eval tval=\$_mkc_opt_${onm}
     if [ "$tval" != "" ]; then
       found=T
@@ -379,10 +429,22 @@ check_option () {
   setdata ${_MKCONFIG_PREFIX} ${nm} "${oval}"
 }
 
+check_echo () {
+  val=$1
+
+  echo "## echo: $val" >&9
+  echo "$val" >&1
+}
+
+check_exit () {
+  echo "## exit" >&9
+  _exitmkconfig 5
+}
+
 _doloadunit () {
   lu=$1
   dep=$2
-  if [ "$dep" = "Y" ]; then
+  if [ "$dep" = Y ]; then
    slu=${lu}
    tag=" (dependency)"
   fi
@@ -394,7 +456,7 @@ _doloadunit () {
     dosubst tlu '-' '_'
     eval "_MKCONFIG_UNIT_${tlu}=Y"
   fi
-  if [ "$dep" = "Y" ]; then
+  if [ "$dep" = Y ]; then
     lu=$slu
     tag=""
   fi
@@ -407,7 +469,7 @@ require_unit () {
     dosubst trqu '-' '_'
     cmd="val=\$_MKCONFIG_UNIT_${trqu}"
     eval $cmd
-    if [ "$val" = "Y" ]; then
+    if [ "$val" = Y ]; then
       echo "   required unit ${rqu} already loaded" >&9
       continue
     fi
@@ -423,12 +485,14 @@ _create_output () {
     exec 8>>${CONFH}
     preconfigfile ${CONFH} >&8
 
-    exec 6<&0 < $VARSFILE
-    while read cfgvar; do
-      getdata val ${_MKCONFIG_PREFIX} $cfgvar
-      output_item ${CONFH} ${cfgvar} "${val}" >&8
-    done
-    exec <&6 6<&-
+    if [ -f $VARSFILE ]; then
+      exec 6<&0 < $VARSFILE
+      while read cfgvar; do
+        getdata val ${_MKCONFIG_PREFIX} $cfgvar
+        output_item ${CONFH} ${cfgvar} "${val}" >&8
+      done
+      exec <&6 6<&-
+    fi
 
     stdconfigfile ${CONFH} >&8
     cat $INC >&8
@@ -472,6 +536,14 @@ main_process () {
   # this allows us to run the while loop in the
   # current shell rather than a subshell.
 
+  # default varsfile.
+  # a main loadunit will override this.
+  # but don't open it unless it is needed.
+  varsfileopen=F
+  if [ "$VARSFILE" = "" -a "${_MKCONFIG_PREFIX}" != "" ]; then
+    VARSFILE="../mkc_none_${_MKCONFIG_PREFIX}.vars"
+  fi
+
   # save stdin in fd 7; open stdin
   exec 7<&0 < ${configfile}
   while read tdatline; do
@@ -484,9 +556,7 @@ main_process () {
           ininclude=0
           resetifs
         else
-          set -f
           echo "${tdatline}" >> $INC
-          set +f
         fi
     else
         case ${tdatline} in
@@ -537,81 +607,25 @@ main_process () {
 
       if [ $doproc -eq 1 ]; then
         case ${tdatline} in
+          command*)
+            _chkconfigfname
+            set $tdatline
+            cmd=$2
+            nm="_command_${cmd}"
+            check_command ${nm} ${cmd}
+            ;;
+          "echo"*)
+            _chkconfigfname
+            set $tdatline
+            shift
+            val=$@
+            check_echo "${val}"
+            ;;
+          "exit")
+            check_exit
+            ;;
           endinclude)
             ;;
-          output*)
-            if [ $inproc -eq 1 ]; then
-              _create_output
-              CONFH=none
-            fi
-            set $tdatline
-            type=$1
-            file=$2
-            case ${file} in
-              none)
-                CONFH=${file}
-                ;;
-              /*)
-                CONFH=${file}
-                ;;
-              *)
-                CONFH="../${file}"
-                ;;
-            esac
-            echo "output-file: ${file}" >&1
-            echo "   config file name: ${CONFH}" >&9
-            inproc=1
-            ;;
-          option-file*)
-            set $tdatline
-            type=$1
-            file=$2
-            case ${file} in
-              /*)
-                OPTIONFILE=${file}
-                ;;
-              *)
-                OPTIONFILE="../${file}"
-                ;;
-            esac
-            echo "option-file: ${file}" >&1
-            echo "   option file name: ${OPTIONFILE}" >&9
-            ;;
-          loadunit*)
-            set $tdatline
-            type=$1
-            file=$2
-            _doloadunit ${file} N
-            if [ "$VARSFILE" = "" -a "${_MKCONFIG_PREFIX}" != "" ]; then
-              VARSFILE="../mkconfig_${_MKCONFIG_PREFIX}.vars"
-            fi
-            exec 8>>$VARSFILE
-            ;;
-          standard)
-            _chkconfigfname
-            standard_checks
-            ;;
-          "set "*|setint*|setstr*)
-            _chkconfigfname
-            set $tdatline
-            type=$1
-            nm=$2
-            if [ "$type" = "setint" -o "$type" = "setstr" ]; then
-              nm="_${type}_$2"
-            fi
-            shift; shift
-            tval=$@
-            check_set ${nm} $type "${tval}"
-            ;;
-          option*)
-            _chkconfigfname
-            set $tdatline
-            optnm=$2
-            shift; shift
-            tval=$@
-            nm="_opt_${optnm}"
-            check_option ${nm} $optnm "${tval}"
-            ;;
           ifoption*|ifnotoption*)
             _chkconfigfname
             set $tdatline
@@ -649,17 +663,100 @@ main_process () {
             ;;
           "endif")
             ;;
-          command*)
-            _chkconfigfname
-            set $tdatline
-            cmd=$2
-            nm="_command_${cmd}"
-            check_command ${nm} ${cmd}
-            ;;
           include)
             _chkconfigfname
             ininclude=1
             ;;
+          loadunit*)
+            set $tdatline
+            type=$1
+            file=$2
+            _doloadunit ${file} N
+            if [ $varsfileopen = T ]; then
+              exec 8>&-
+              varsfileopen=F
+            fi
+            if [ "${_MKCONFIG_PREFIX}" != "" ]; then
+              VARSFILE="../mkc_${CONFHTAG}_${_MKCONFIG_PREFIX}.vars"
+            fi
+            ;;
+          option-file*)
+            set $tdatline
+            type=$1
+            file=$2
+            case ${file} in
+              /*)
+                OPTIONFILE=${file}
+                ;;
+              *)
+                OPTIONFILE="../${file}"
+                ;;
+            esac
+            echo "option-file: ${file}" >&1
+            echo "   option file name: ${OPTIONFILE}" >&9
+            ;;
+          option*)
+            _chkconfigfname
+            set $tdatline
+            optnm=$2
+            shift; shift
+            tval=$@
+            nm="_opt_${optnm}"
+            check_option ${nm} $optnm "${tval}"
+            ;;
+          output*)
+            if [ $varsfileopen = T ]; then
+              exec 8>&-
+              varsfileopen=F
+            fi
+            if [ $inproc -eq 1 ]; then
+              _create_output
+              CONFH=none
+              CONFHTAG=none
+              CONFHTAGUC=NONE
+            fi
+            set $tdatline
+            type=$1
+            file=$2
+            case ${file} in
+              none)
+                CONFH=${file}
+                ;;
+              /*)
+                CONFH=${file}
+                ;;
+              *)
+                CONFH="../${file}"
+                ;;
+            esac
+            echo "output-file: ${file}" >&1
+            echo "   config file name: ${CONFH}" >&9
+            file=`echo $file | sed -e 's,.*/,,'`
+            file=`echo $file | sed -e 's/\..*//'`
+            CONFHTAG=$file
+            CONFHTAGUC=$file
+            toupper CONFHTAGUC
+            if [ "${_MKCONFIG_PREFIX}" != "" ]; then
+              VARSFILE="../mkc_${CONFHTAG}_${_MKCONFIG_PREFIX}.vars"
+            fi
+            inproc=1
+            ;;
+          standard)
+            _chkconfigfname
+            standard_checks
+            ;;
+          "set "*|setint*|setstr*)
+            _chkconfigfname
+            set $tdatline
+            type=$1
+            nm=$2
+            if [ "$type" = "setint" -o "$type" = "setstr" ]; then
+              nm="_${type}_$2"
+            fi
+            shift; shift
+            tval=$@
+            check_set ${nm} $type "${tval}"
+            ;;
           *)
             _chkconfigfname
             set $tdatline
@@ -678,7 +775,10 @@ main_process () {
   # reset the file descriptors back to the norm.
   # set stdin to saved fd 7; close fd 7
   exec <&7 7<&-
-  exec 8>&-
+  if [ $varsfileopen = T ]; then
+    exec 8>&-
+    varsfileopen=F
+  fi
 
   _savecache     # save the cache file.
   _create_output
@@ -698,8 +798,6 @@ defaults:
 mkconfigversion
 
 unset GREP_OPTIONS
-unset DI_ARGS
-unset DI_FMT
 unset ENV
 unalias sed > /dev/null 2>&1
 unalias grep > /dev/null 2>&1
@@ -707,7 +805,6 @@ unalias ls > /dev/null 2>&1
 unalias rm > /dev/null 2>&1
 LC_ALL=C
 export LC_ALL
-
 clearcache=0
 while test $# -gt 1; do
   case $1 in
@@ -752,7 +849,11 @@ cd $_MKCONFIG_TMP
 LOG="../$LOG"
 REQLIB="../$REQLIB"
 CACHEFILE="../$CACHEFILE"
+VARSFILE="../$VARSFILE"
 OPTIONFILE="../$OPTIONFILE"
+CONFH=none
+CONFHTAG=none
+CONFHTAGUC=NONE
 
 if [ $clearcache -eq 1 ]; then
   rm -f $CACHEFILE > /dev/null 2>&1
@@ -771,6 +872,21 @@ echo "has append: ${shhasappend}" >&9
 echo "has math: ${shhasmath}" >&9
 echo "has upper: ${shhasupper}" >&9
 
+locatecmd awkcmd awk
+locatecmd nawkcmd nawk
+locatecmd gawkcmd gawk
+locatecmd mawkcmd mawk
+if [ "$nawkcmd" != "" ]; then
+  awkcmd=$nawkcmd
+fi
+if [ "$mawkcmd" != "" ]; then
+  awkcmd=$mawkcmd
+fi
+if [ "$gawkcmd" != "" ]; then
+  awkcmd=$gawkcmd
+fi
+echo "awk: $awkcmd" >&9
+
 echo "$0 ($shell) using $configfile"
 
 main_process $configfile
