@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# $Id: mklink.sh,v 1.5 2010-12-29 13:09:06-08 bll Exp $
+# $Id: mklink.sh,v 1.6 2011-09-16 10:30:02-07 bll Exp $
 #
 # Copyright 2010 Brad Lanam Walnut Creek, CA USA
 #
@@ -51,7 +51,7 @@ done
 OUTFLAG="-o "
 DC_LINK=
 case ${comp} in
-  *dmd)
+  *dmd|*dmd2|*ldc|*ldc2)   # catches ldmd, ldmd2 also
     OUTFLAG="-of"
     DC_LINK=-L
     ;;
