@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# $Id: runtests.sh,v 1.63 2010-12-28 20:46:18-08 bll Exp $
+# $Id: runtests.sh,v 1.75 2011-10-07 14:05:51-07 bll Exp $
 # $Source: /home/bll/DI/di/mkconfig/RCS/runtests.sh,v $
 #
 # Copyright 1994-2010 Brad Lanam, Walnut Creek, CA
@@ -15,6 +15,8 @@
 #    5 - stdout (as 1 is directed to the log)
 #
 
+set -f # global
+
 DOPERL=T
 TESTORDER=test_order
 SUBDIR=F
@@ -44,8 +46,6 @@ if [ $# -lt 1 ]; then
 fi
 
 unset GREP_OPTIONS
-unset DI_ARGS
-unset DI_FMT
 unset ENV
 unalias sed > /dev/null 2>&1
 unalias grep > /dev/null 2>&1
@@ -62,11 +62,13 @@ getlistofshells () {
   echo "## paths: $_pthlist" >&8
 
   tshelllist=""
+  inodelist=""
   for d in $_pthlist; do
     for s in $tryshell ; do
       rs=$d/$s
-      if [ -h $rs ]; then
+      if [ -x $rs ]; then
         while [ -h $rs ]; do
+          ors=$rs
           rs="`ls -l $rs | sed 's/.* //'`"
           case $rs in
             /*)
@@ -75,44 +77,86 @@ getlistofshells () {
               rs="$d/$rs"
               ;;
           esac
+
+          # /etc/alternatives/xxx has some weird names w/dots.
+          # all ksh* are ksh
+          # anything not ksh/bash/zsh is sh
+          # if the name is sh->bash or sh->ksh; don't follow symlink.
+          ts=`echo $rs | sed -e 's,.*[/\.],,' \
+              -e 's/ksh88/ksh/' -e 's/ksh93/ksh/' `
+          tts=`echo $s | sed -e 's/ksh88/ksh/' -e 's/ksh93/ksh/' `
+          if [ $ts != ksh -a $ts != bash -a $ts != zsh ]; then
+            ts=sh
+          fi
+          if [ $ts != ksh -a $tts != bash -a $tts != zsh ]; then
+            tts=sh
+          fi
+          if [ $ts != $tts ]; then
+            rs=$ors
+            break
+          fi
+
+          rs=`echo $rs | sed 's,/[^/]*/\.\./,/,'`
           rs=`echo $rs | sed 's,/[^/]*/\.\./,/,'`
           rs=`echo $rs | sed 's,/[^/]*/\.\./,/,'`
           rs=`echo $rs | sed 's,/[^/]*/\.\./,/,'`
         done
-      fi
-      if [ -x $rs ]; then
-        cmd="$rs -c \". $_MKCONFIG_DIR/shellfuncs.sh;getshelltype;echo \\\$shell\""
-        shell=`eval $cmd`
-        echo "  found: $rs ($shell)" >&8
-        case $shell in
-          pdksh)
-            echo "    skip" >&8
-            ;;
-          *)
-            tshelllist="${tshelllist}
+
+        inode=`ls -i ${rs} | sed -e 's/^ *//' -e 's/ .*//'`
+        inode=${inode}${s}   # append shell type also
+        found=F
+        for i in $inodelist; do
+          if [ "$inode" = "${i}" ]; then
+            found=T
+            break
+          fi
+        done
+        if [ $found = T ]; then
+          continue
+        fi
+
+        if [ -x $rs ]; then
+          cmd="$rs -c \". $_MKCONFIG_DIR/shellfuncs.sh;getshelltype $rs echo\""
+          set `eval $cmd`
+          dispshell=$1
+          echo "  found: $rs ($dispshell)" >&8
+          case $dispshell in
+            *)
+              tshelllist="${tshelllist}
 $rs"
-            ;;
-        esac
-      fi
+              if [ "${inode}" != ${s} ]; then
+                inodelist="${inodelist}
+${inode}"
+              fi
+              ;;
+          esac
+        fi # if executable
+      fi  # if there is a file
     done
   done
-  tshelllist=`echo "$tshelllist" | sort -u`
 
+  tshelllist=`echo "$tshelllist" | sort -u`
   systype=`uname -s`
   shelllist=""
   for s in $tshelllist; do
     echo ${EN} "  check $s${EC}" >&8
-    echo ${EN} "$s${EC}"
-    cmd="$s -c \". $_MKCONFIG_DIR/shellfuncs.sh;TSHELL=$s;chkshell\""
+    echo ${EN} "   $s${EC}"
+    cmd="$s -c \". $_MKCONFIG_DIR/shellfuncs.sh;TSHELL=$s;chkshell echo\""
     eval $cmd >&8 2>&1
-    if [ $? -eq 0 ]; then
+    rc=$?
+    cmd="$s -c \". $_MKCONFIG_DIR/shellfuncs.sh;TSHELL=$s;getshelltype $s echo\""
+    set `eval $cmd`
+    dispshell=$1
+    shift
+    shvers=$@
+    if [ $rc -eq 0 ]; then
       echo " ok" >&8
-      echo ${EN} "(ok) ${EC}"
       shelllist="${shelllist} $s"
+      echo " [$dispshell $shvers] (ok)"
     else
       echo " ng" >&8
-      echo ${EN} "(ng) ${EC}"
-      echo $chkmsg >&8
+      echo " : $chkmsg" >&8
+      echo " [$dispshell $shvers] (ng)"
     fi
   done
 }
@@ -120,7 +164,7 @@ $rs"
 runshelltest () {
   stag=""
   if [ "$_MKCONFIG_SHELL" != "" ]; then
-    stag=".${scount}_${shell}"
+    stag=".${scount}_${dispshell}"
   fi
   TSTRUNLOG=${_MKCONFIG_TSTRUNTMPDIR}/${tbase}.log${stag}
   > $TSTRUNLOG
@@ -136,7 +180,7 @@ runshelltest () {
 
   cd $_MKCONFIG_TSTRUNTMPDIR
   if [ "$_MKCONFIG_SHELL" != "" ]; then
-    echo ${EN} " ${ss}${EC}"
+    echo ${EN} " ${dispshell}${EC}"
   fi
   targ=$arg
   if [ "$arg" != "" ]; then
@@ -166,10 +210,22 @@ runshelltest () {
 _MKCONFIG_RUNTOPDIR=`pwd`
 export _MKCONFIG_RUNTOPDIR
 mypath=`echo $0 | sed -e 's,/[^/]*$,,'`
+if [ "$mypath" = "runtests.sh" ]; then
+  mypath=.
+fi
 _MKCONFIG_DIR=`(cd $mypath;pwd)`
 export _MKCONFIG_DIR
 . ${_MKCONFIG_DIR}/shellfuncs.sh
 
+_MKC_ONCE=0
+export _MKC_ONCE
+_MKC_SH=1
+export _MKC_SH
+_MKC_PL=2
+export _MKC_PL
+_MKC_SH_PL=3
+export _MKC_SH_PL
+
 doshelltest $0 $@
 if [ $SUBDIR = "F" ]; then
   setechovars
@@ -212,7 +268,7 @@ TMPORDER=test_order.tmp
 > $TMPORDER
 if [ "$teststorun" = "" ]; then
   if [ ! -f "$TESTORDER" ]; then
-    ls -1d *.d *.sh 2>/dev/null | sed -e 's/\.sh$//' -e 's/^/1 ' >> $TMPORDER
+    ls -1d *.d *.sh 2>/dev/null | sed -e 's/\.sh$//' -e 's/^/1/' >> $TMPORDER
   else
     sort -n $TESTORDER >> $TMPORDER
   fi
@@ -235,11 +291,24 @@ exec 8>>$MAINLOG
 
 if [ $SUBDIR = "F" ]; then
   echo "## locating valid shells"
-  echo ${EN} "   ${EC}"
   getlistofshells
-  echo ""
 fi
 
+locatecmd awkcmd awk
+locatecmd nawkcmd nawk
+locatecmd gawkcmd gawk
+locatecmd mawkcmd mawk
+if [ "$nawkcmd" != "" ]; then
+  awkcmd=$nawkcmd
+fi
+if [ "$mawkcmd" != "" ]; then
+  awkcmd=$mawkcmd
+fi
+if [ "$gawkcmd" != "" ]; then
+  awkcmd=$gawkcmd
+fi
+echo "awk: $awkcmd" >&8
+
 export shelllist
 grc=0
 count=0
@@ -318,41 +387,35 @@ while read tline; do
 
   dt=`date`
   arg="mkconfig.sh"
-  suffix=""
-  if [ -f ${tbase}.mkshpl ]; then
-    suffix="_sh"
-  else
-    suffix=""
-  fi
 
   scount=""
   echo ${EN} "$tbase ...${EC}"
   echo ${EN} "$tbase ...${EC}" >&8
-  _MKCONFIG_TSTRUNTMPDIR=$_MKCONFIG_RUNTMPDIR/${tbase}${suffix}
+  _MKCONFIG_TSTRUNTMPDIR=$_MKCONFIG_RUNTMPDIR/${tbase}
   export _MKCONFIG_TSTRUNTMPDIR
   mkdir ${_MKCONFIG_TSTRUNTMPDIR}
   if [ -f $tconfig ]; then
     cp $tconfig $_MKCONFIG_TSTRUNTMPDIR/$tconfh
   fi
-  $_MKCONFIG_RUNTESTDIR/$tf -d
-  $_MKCONFIG_RUNTESTDIR/$tf -d >&8
+  tfdisp=`$_MKCONFIG_RUNTESTDIR/$tf -d`
+  echo ${EN} " ${tfdisp}${EC}"
+  echo ${EN} " ${tfdisp}${EC}" >&8
+  $_MKCONFIG_RUNTESTDIR/$tf -q
+  runshpl=$?
 
-  if [ -f ${tbase}.mksh -o -f ${tbase}.mkshpl ]; then
+  if [ $runshpl -eq $_MKC_SH -o $runshpl -eq $_MKC_SH_PL ]; then
     echo ${EN} " ...${EC}"
     echo ${EN} " ...${EC}" >&8
     src=0
     scount=1
     for s in $shelllist; do
       unset _shell
-      unset shell
-      cmd="$s -c \". $_MKCONFIG_DIR/shellfuncs.sh;getshelltype;echo \\\$shell\""
-      ss=`eval $cmd`
-      if [ "$ss" = "sh" ]; then
-        ss=`echo $s | sed 's,.*/,,'`
-      fi
+      unset dispshell
+      cmd="$s -c \". $_MKCONFIG_DIR/shellfuncs.sh;getshelltype $s echo\""
+      set `eval $cmd`
+      dispshell=$1
       _MKCONFIG_SHELL=$s
       export _MKCONFIG_SHELL
-      shell=$ss
 
       runshelltest
       rc=$?
@@ -360,7 +423,7 @@ while read tline; do
       domath scount "$scount + 1"
 
       unset _shell
-      unset shell
+      unset dispshell
       unset _MKCONFIG_SHELL
     done
   else
@@ -393,7 +456,8 @@ while read tline; do
   fi
   domath count "$count + 1"
 
-  if [ "$DOPERL" = "T" -a \( -f ${tbase}.mkshpl -o -f ${tbase}.mkpl \) ]; then
+  if [ "$DOPERL" = "T" -a \
+       \( $runshpl -eq $_MKC_PL -o $runshpl -eq $_MKC_SH_PL \) ]; then
     _MKCONFIG_TSTRUNTMPDIR=$_MKCONFIG_RUNTMPDIR/${tbase}_pl
     export _MKCONFIG_TSTRUNTMPDIR
     mkdir ${_MKCONFIG_TSTRUNTMPDIR}
@@ -408,8 +472,8 @@ while read tline; do
     echo "####" >&9
     echo ${EN} "$tbase ...${EC}"
     echo ${EN} "$tbase ...${EC}" >&8
-    $_MKCONFIG_RUNTESTDIR/$tf -d
-    $_MKCONFIG_RUNTESTDIR/$tf -d >&8
+    echo ${EN} " ${tfdisp}${EC}"
+    echo ${EN} " ${tfdisp}${EC}" >&8
     echo ${EN} " ... perl${EC}"
     echo ${EN} " ... perl${EC}" >&8
     echo "## Using mkconfig.pl " >&9
@@ -419,7 +483,7 @@ while read tline; do
 
     cd $_MKCONFIG_TSTRUNTMPDIR
     # dup stdout to 5; redirect stdout to 9; redirect stderr to new 1.
-    $_MKCONFIG_RUNTESTDIR/$tf perl $_MKCONFIG_DIR/mkconfig.pl 5>&1 >&9 2>&1
+    $_MKCONFIG_RUNTESTDIR/$tf none $_MKCONFIG_DIR/mkconfig.pl 5>&1 >&9 2>&1
     rc=$?
     cd $_MKCONFIG_RUNTESTDIR
 
