@@ -1,18 +1,31 @@
 #!/bin/sh
 #
-# $Id: shellfuncs.sh,v 1.79 2010-07-22 09:56:14-07 bll Exp $
-# $Revision: 1.79 $
+# $Id: shellfuncs.sh,v 1.86 2011-10-07 14:06:17-07 bll Exp $
+# $Revision: 1.86 $
 #
 # Copyright 2010 Brad Lanam Walnut Creek, CA, USA
 #
 
-read _MKCONFIG_VERSION < ${_MKCONFIG_DIR}/VERSION
+if [ ! -f "${_MKCONFIG_DIR}/VERSION" ]; then
+  echo "Unable to locate ${_MKCONFIG_DIR}/VERSION."
+  echo "Not a valid mkconfig installation."
+  exit 1
+fi
+
+# dash on Mandriva 2011 segfaults intermittently when:
+#   read _MKCONFIG_VERSION < ${_MKCONFIG_DIR}/VERSION
+#   export _MKCONFIG_VERSION
+# was used.
+_MKCONFIG_VERSION=`cat ${_MKCONFIG_DIR}/VERSION`
 export _MKCONFIG_VERSION
 
 # posh set command doesn't output correct data; don't bother with it.
-# pdksh is often broken; avoid it.
-# zsh set output not correct.
-tryshell="ash bash dash ksh mksh sh sh5"
+# yash has quoting/backquote issues
+# zsh is not bourne shell compatible
+#   set | grep can be fixed with: set | strings | grep
+#   and 'emulate ksh' can be set in mkconfig.sh, but there
+#   are more issues, and I'm not interested in tracking them down.
+tryshell="ash bash dash ksh ksh88 ksh93 mksh pdksh sh sh5"
 
 mkconfigversion () {
   echo "mkconfig version ${_MKCONFIG_VERSION}"
@@ -68,7 +81,7 @@ test_math () {
 
 test_upper () {
   shhasupper=0
-  (eval 'typeset -u var;var=x;test z$var = zX') 2>/dev/null
+  (eval 'typeset -u xuvar;xuvar=x;test z$xuvar = zX') 2>/dev/null
   if [ $? -eq 0 ]; then
     shhasupper=1
     eval 'toupper () { ucvar=$1; typeset -u uval; eval "uval=\${$ucvar};$ucvar=\$uval"; }'
@@ -79,7 +92,7 @@ test_upper () {
 
 test_lower () {
   shhaslower=0
-  (eval 'typeset -l var;var=X;test z$var = zx') 2>/dev/null
+  (eval 'typeset -l xuvar;xuvar=X;test z$xuvar = zx') 2>/dev/null
   if [ $? -eq 0 ]; then
     shhaslower=1
     eval 'tolower () { lcvar=$1; typeset -l lval; eval "lval=\${$lcvar};$lcvar=\$lval"; }'
@@ -96,8 +109,22 @@ testshcapability () {
 }
 
 getshelltype () {
-  shell=${_shell:-sh}   # unknown or old
-  baseshell=${_shell:-sh}
+  if [ "$1" != "" ]; then
+    trs=$1
+    shift
+  fi
+  gstecho=F
+  if [ "$1" = echo ]; then
+    gstecho=T
+  fi
+
+  baseshell=${_shell:-sh} # unknown or old
+  shell=${_shell:-sh}     # unknown or old
+  if [ "$trs" != "" ]; then
+    dispshell=`echo $trs | sed -e 's,.*/,,'`
+  else
+    dispshell=$shell
+  fi
   ( eval 'echo ${.sh.version}' ) >/dev/null 2>&1
   if [ $? -eq 0 ]; then
     eval 'KSH_VERSION=${.sh.version}'
@@ -105,6 +132,7 @@ getshelltype () {
   if [ "$KSH_VERSION" != "" ]; then
     shell=ksh
     baseshell=ksh
+    shvers=$KSH_VERSION
     case $KSH_VERSION in
       *PD*)
         shell=pdksh
@@ -120,20 +148,35 @@ getshelltype () {
         ;;
     esac
   elif [ "$BASH_VERSION" != "" ]; then
-    baseshell=bash
+    shvers=$BASH_VERSION
     ver=`echo $BASH_VERSION | sed 's/\..*//'`
     shell=bash${ver}
+    baseshell=bash
   elif [ "$ZSH_VERSION" != "" ]; then
-    baseshell=zsh
+    shvers=$ZSH_VERSION
     shell=zsh
+    baseshell=zsh
   elif [ "$POSH_VERSION" != "" ]; then
-    baseshell=posh
+    shvers=$POSH_VERSION
     shell=posh
+    baseshell=posh
+  elif [ "$YASH_VERSION" != "" ]; then
+    shvers=$YASH_VERSION
+    shell=yash
+    baseshell=yash
   fi
 
+  if [ $dispshell = sh -a $dispshell != $shell ]; then
+    dispshell="$dispshell-$shell"
+  elif [ $dispshell = $baseshell ]; then
+    dispshell=$shell
+  fi
   # can try --version, but don't really know the path
   # of the shell running us; can't depend on $SHELL.
   # and it only works for bash and some versions of ksh.
+  if [ $gstecho = T ]; then
+    echo $dispshell $shvers
+  fi
 }
 
 doshelltest () {
@@ -149,11 +192,10 @@ doshelltest () {
     fi
   fi
 
-  getshelltype
-
+  getshelltype  # for display of error below
   chkshell
   if [ $? -ne 0 ]; then
-    echo "The shell in use ($shell) does not have the correct functionality:" >&2
+    echo "The shell in use ($dispshell) does not have the correct functionality:" >&2
     echo $chkmsg >&2
     echo "Please try another shell.
 _MKCONFIG_SHELL can be set to the path of another shell
@@ -182,14 +224,20 @@ locatecmd () {
 # function to make sure the shell has
 # some basic capabilities w/o weirdness.
 chkshell () {
+  doecho=F
+  if [ "$1" = "echo" ]; then
+    doecho=T
+  fi
 
   grc=0
 
   TMP=chksh$$
   chkmsg=""
   # test to make sure the set command works properly
+  # some shells output xyzzy=abc def
   # some shells output xyzzy='abc def'
   # some shells output xyzzy=$'abc def' (ok; handled in mkconfig.sh)
+  # yash does this correctly, but has quoting/backquote issues
   (
     cmd='xyzzy="abc def"; val=`set | grep "^xyzzy"`; test "$val" = "xyzzy=abc def"'
     eval $cmd 2>/dev/null
@@ -198,10 +246,8 @@ chkshell () {
     fi
     cmd="xyzzy=\"abc def\"; val=\`set | grep \"^xyzzy\"\`; test \"\$val\" = \"xyzzy='abc def'\" -o \"\$val\" = \"xyzzy=\\$'abc def'\""
     eval $cmd 2>/dev/null
-    if [ $? -eq 0 ]; then
-      exit 0
-    fi
-    exit 1
+    rc=$?
+    exit $rc
   )
   rc=$?
   if [ $rc -ne 0 ]; then
@@ -210,11 +256,28 @@ chkshell () {
   'set' output not x=a b or x='a b' or x=\$'a b'."
   fi
 
+  # test to make sure the 'set -f' command is supported.
+  (
+    cmd='set -f'
+    eval $cmd 2>/dev/null
+    rc=$?
+    if [ $rc -eq 0 ]; then
+      exit 0
+    fi
+    exit $rc
+  )
+  rc=$?
+  if [ $rc -ne 0 ]; then
+    grc=$rc
+    chkmsg="${chkmsg}
+  'set -f' not supported"
+  fi
+
   if [ "$TSHELL" != "" ]; then
     # test for -n not supported.
     (
       rm -f $TMP $TMP.out > /dev/null 2>&1
-      echo 'exit 1' > $TMP
+      echo 'while test $# -gt 1; do echo $1; shift; done; exit 1' > $TMP
       chmod a+rx $TMP
       cmd="$TSHELL -n $TMP;echo \$? > $TMP.out"
       eval $cmd &
@@ -237,6 +300,10 @@ chkshell () {
     fi
   fi
 
+  if [ $doecho = "T" ]; then
+    echo $chkmsg
+  fi
+
   return $grc
 }
 
@@ -288,6 +355,7 @@ initifs () {
 }
 
 setifs () {
+  # just newline for parsing include section
   IFS="
 "
 }
